/*
 * GNOME Contacts
 */

/* The link suggestion widget */
.contacts-link-suggestion {
  border-top: 1px solid @borders;
  background-color: shade(@theme_bg_color, 0.9);
}

  .contacts-link-suggestion avatar {
    margin: 12px;
  }

.contacts-avatar-popover .contact-display-name {
  font-size: 20px;
}

/* Used to provide a little cutout around an overlayed circular button */
.contacts-cutout-button {
  background-color: @window_bg_color;
  border-radius: 9999px;
  padding: 2px;
}

/*         Contact List        */
/* --------------------------- */

.contact-list header {
  padding: 0 6px;
}

.contact-list-row {
  padding: 6px 0;
}

/* Contact Sheet/Editor common */
/* --------------------------- */

.contacts-sheet-container,
.contacts-contact-editor-container {
  margin: 24px 12px 24px 12px;
}

.contacts-sheet-header,
.contacts-editor-header {
  margin: 0 0 12px 0;
}

.contacts-sheet-property,
.contacts-editor-property {
  margin: 9px 6px;
}

/* Contact Sheet-related CSS class */
/* ------------------------------- */

.contacts-sheet {
}

.contacts-sheet-property {
}


/* Contact Editor-related CSS classes */
/* ---------------------------------- */

.contacts-editor-header-name {
  /* The name in the sheet gets some margin from .title-1,
   * so do something similar here to prevent the name "jumping" */
  margin: 6px 0;
}

/* Common class for all widgets editing a property  */
.contacts-editor-property {
}

  .contacts-editor-property .contacts-property-icon {
    margin: 12px 12px;
  }

  .contacts-editor-property button.contacts-editor-main-widget {
    padding: 10px 0;
  }

.contacts-editor-address,
.contacts-editor-role {
  padding-top: 6px;
  padding-bottom: 6px;
}

/* Widget to edit a birthday (_not_ the row itself) */
.contacts-editor-birthday {
  margin: 12px;
}

.contacts-qr-code-dialog-qr-image {
  background: #fff;
  padding: 12px;
}

.favorite-button {
  font-weight: normal;
}
